@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2007 the original author or authors.
+ * Copyright 2002-2008 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -479,10 +479,10 @@ public final class JSONUtils {
     * @return A String correctly formatted for insertion in a JSON text.
     */
    public static String quote( String string ) {
-      if( isFunction( string ) ){
+      if( isFunction( string ) ) {
          return string;
       }
-      if( string == null || string.length() == 0 ){
+      if( string == null || string.length() == 0 ) {
          return "\"\"";
       }
 
@@ -490,50 +490,70 @@ public final class JSONUtils {
       char c = 0;
       int i;
       int len = string.length();
-      StringBuffer sb = new StringBuffer( len + 4 );
+      StringBuilder sb = new StringBuilder( len * 2 );
       String t;
-
+      char[] chars = string.toCharArray();
+      char[] buffer = new char[1030];
+      int bufferIndex = 0;
       sb.append( '"' );
-      for( i = 0; i < len; i += 1 ){
+      for( i = 0; i < len; i += 1 ) {
+         if( bufferIndex > 1024 ) {
+            sb.append( buffer, 0, bufferIndex );
+            bufferIndex = 0;
+         }
          b = c;
-         c = string.charAt( i );
-         switch( c ){
+         c = chars[i];
+         switch( c ) {
             case '\\':
             case '"':
-               sb.append( '\\' );
-               sb.append( c );
+               buffer[bufferIndex++] = '\\';
+               buffer[bufferIndex++] = c;
                break;
             case '/':
-               if( b == '<' ){
-                  sb.append( '\\' );
+               if( b == '<' ) {
+                  buffer[bufferIndex++] = '\\';
                }
-               sb.append( c );
-               break;
-            case '\b':
-               sb.append( "\\b" );
-               break;
-            case '\t':
-               sb.append( "\\t" );
-               break;
-            case '\n':
-               sb.append( "\\n" );
-               break;
-            case '\f':
-               sb.append( "\\f" );
-               break;
-            case '\r':
-               sb.append( "\\r" );
+               buffer[bufferIndex++] = c;
                break;
             default:
-               if( c < ' ' ){
-                  t = "000" + Integer.toHexString( c );
-                  sb.append( "\\u" )
-                        .append( t.substring( t.length() - 4 ) );
-               }else{
-                  sb.append( c );
+               if( c < ' ' ) {
+                  switch( c ) {
+                     case '\b':
+                        buffer[bufferIndex++] = '\\';
+                        buffer[bufferIndex++] = 'b';
+                        break;
+                     case '\t':
+                        buffer[bufferIndex++] = '\\';
+                        buffer[bufferIndex++] = 't';
+                        break;
+                     case '\n':
+                        buffer[bufferIndex++] = '\\';
+                        buffer[bufferIndex++] = 'n';
+                        break;
+                     case '\f':
+                        buffer[bufferIndex++] = '\\';
+                        buffer[bufferIndex++] = 'f';
+                        break;
+                     case '\r':
+                        buffer[bufferIndex++] = '\\';
+                        buffer[bufferIndex++] = 'r';
+                        break;
+                     default:
+                        t = "000" + Integer.toHexString( c );
+                        int tLength = t.length();
+                        buffer[bufferIndex++] = '\\';
+                        buffer[bufferIndex++] = 'u';
+                        buffer[bufferIndex++] = t.charAt( tLength - 4 );
+                        buffer[bufferIndex++] = t.charAt( tLength - 3 );
+                        buffer[bufferIndex++] = t.charAt( tLength - 2 );
+                        buffer[bufferIndex++] = t.charAt( tLength - 1 );
+                  }
+               } else {
+                  buffer[bufferIndex++] = c;
                }
          }
       }
+      sb.append( buffer, 0, bufferIndex );
       sb.append( '"' );
       return sb.toString();
    }
